/*
	MacBinHandler.cpp
	
	Copyright 1996, Jake Hamby
*/

#include "MacBinHandler.h"

MacBinHandler::MacBinHandler() {
}

//---------------------
// Open the original value and sanity check the header
//---------------------
long MacBinHandler::open_ref(record_ref oldref) {
	long returnValue;

	oldfile.SetRef(oldref);
	if(oldfile.Open(B_READ_ONLY) < B_NO_ERROR)
		return M_CANT_OPEN;			// Can't open file
	
	if(oldfile.Read(hdrBuffer, 128) != 128)
		returnValue = M_READ_ERROR;	// Can't read header
	else if(!SanityCheck())
		returnValue = M_CRC_ERROR;	// Not a MacBinary file!
	else	
		returnValue = M_NO_ERROR;

	oldfile.Close();
	return returnValue;
}

//---------------------
// This procedure is probably really slow, but we only
// need to CRC the header (124 bytes)
//---------------------
#define CCITT_CRC_GEN  0x1021

unsigned short MacBinHandler::CalcCRC(unsigned char *buf, int size) {
	unsigned short crc=0, dataByte;
	
	while (size--) {
		dataByte = *buf++ << 8;
		for (int i=8; i>0; i--) {
			if ((dataByte ^ crc) & 0x8000)
				crc = (crc << 1) ^ CCITT_CRC_GEN;
			else
				crc <<= 1;
			dataByte <<= 1;
		}
	}
	return(crc);
}

#define MACBINARY_VERSION 129

#include <stdio.h>

long MacBinHandler::SanityCheck() {
	bool looksGood;
	int i;
	
	// First check version number and first zero-fill byte
	looksGood = (hdrBuffer[OLD_V_N]==0) && (hdrBuffer[ZERO_FILL_I]==0);
	// If filename length looks reasonable
	if (hdrBuffer[LEN_F_N] > 0 && hdrBuffer[LEN_F_N] < 64 && looksGood) {
		// Transfer filename
		memcpy(hdr.fName, &hdrBuffer[FILENAME], hdrBuffer[LEN_F_N]+1);

		// Transfer type/creator info
		memcpy(&hdr.finderInfo.type, &hdrBuffer[FILE_TYPE], 4);
		memcpy(&hdr.finderInfo.creator, &hdrBuffer[FILE_CREA], 4);

		// Transfer data fork length
		memcpy(&hdr.lenDF, &hdrBuffer[LEN_D_FORK], 4);

		// Transfer CRC
		memcpy(&hdr.crc, &hdrBuffer[CRC], 2);
	
		// Check for MacBinary II
		looksGood = (CalcCRC(hdrBuffer, 124) == hdr.crc) &&
					(hdrBuffer[MBII_VERS_READ] == MACBINARY_VERSION);
	
		// If that fails, check again for MacBinary I (untested!)
		if (!looksGood && (hdrBuffer[ZERO_FILL_II] == 0)) {
			for (i=101; hdrBuffer[i]==0 && i<126; i++);
			// If i>=126, then those locations are zero-padded, so
			// it is indeed MacBinary I.
			looksGood = (i >= 126);
		}
	} else {
		looksGood = FALSE;
	}
	return looksGood;	
}

//---------------------
// Perform the actual conversion
//---------------------

long MacBinHandler::convert(BFile *newfile) {
	int bytesread, bytetotal=0;

	newfile->SetTypeAndApp(hdr.finderInfo.type, hdr.finderInfo.creator);

	if(oldfile.Open(B_READ_ONLY) < B_NO_ERROR)
		return M_CANT_OPEN;

	if(newfile->Open(B_EXCLUSIVE) < B_NO_ERROR) {
		oldfile.Close();
		return M_CANT_OPEN;
	}

    oldfile.Seek(128, B_SEEK_TOP);
    while((bytetotal < hdr.lenDF) &&
			(bytesread=oldfile.Read(copybuffer, 65536))) {
                if((bytetotal+bytesread) > hdr.lenDF)
                        bytesread = hdr.lenDF - bytetotal;
                newfile->Write(copybuffer, bytesread);
                bytetotal += bytesread;
    }
	oldfile.Close();
	newfile->Close();
	if (bytetotal != hdr.lenDF)		// Couldn't transfer entire file
		return M_READ_ERROR;
	else
		return M_NO_ERROR; 		
}
