/*
	MacBinHandler.h
	
	Copyright 1996, Jake Hamby
*/

#ifndef MACBIN_HANDLER_H
#define MACBIN_HANDLER_H
#include <StorageKit.h>

typedef struct FInfo {
		long type, creator;
};

// Note:  The real MacBinary header is 128 bytes long and contains many more
// fields.  However, I removed the fields I didn't need, since you must copy
// each field into the struct manually using the byte offsets in the #defines
// below.  This program isn't portable to machines with a different byte
// ordering (e.g. x86) by the way.
struct MacBinaryHeader {
        char            fName[68];              /* file name (4 extra characters
													so we can add ".new" */
        FInfo           finderInfo;             /* Original Finder information record */
        long            lenDF;                  /* length of data fork */
        unsigned short  crc; 
};

/*
 *      MacBinary II Header #defines
 */

#define OLD_V_N                 0
#define LEN_F_N                 1
#define FILENAME                2
#define FILE_TYPE               65
#define FILE_CREA               69
#define ORIG_F_FLAGS    		73
#define ZERO_FILL_I             74
#define FILE_V_POS              75
#define FILE_H_POS              77
#define WIN_FOL_ID              79
#define PROTECTED               81
#define ZERO_FILL_II    		82
#define LEN_D_FORK              83
#define LEN_R_FORK              87
#define F_CREA_DATE             91
#define F_MOD_DATE              95
#define LEN_GET_INFO    		99
#define FINDER_FLAGS    		101
#define LEN_TOT_F               116
#define LEN_SEC_H               120
#define MBII_VERS_UP    		122
#define MBII_VERS_READ  		123
#define CRC                     124 

class MacBinHandler {
public:
	MacBinHandler();
	long open_ref(record_ref oldref); // Open file & sanity check
	char* get_filename() {				// Get embedded filename
		return hdr.fName;
	}
	long convert(BFile *newfile);  // Save data fork to new file
private:
	unsigned short CalcCRC(unsigned char *buf, int size);
	long SanityCheck();
	BFile oldfile;
	MacBinaryHeader hdr;			// processed MacBinary header
	unsigned char hdrBuffer[128];	// raw MacBinary header buffer
	char copybuffer[65536];			// 64k buffer for copying
};

#define M_NO_ERROR  0
#define M_CANT_OPEN -1
#define M_READ_ERROR -2
#define M_CRC_ERROR -3
#define M_WRITE_ERROR -4

#endif