/*
	StripMacBinWindow.cpp
	
	Copyright 1996, Jake Hamby
*/

#include "StripMacBinary.h"
#include "StripMacBinWindow.h"

StripMacBinWindow::StripMacBinWindow(BRect frame)
                                : BWindow(frame, "StripMacBinary Status", B_TITLED_WINDOW,
                                B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	frame.OffsetTo(B_ORIGIN);
    myView = new StripMacBinView(frame);
    AddChild(myView);

	myMacBinHandler = new MacBinHandler();
}

StripMacBinWindow::~StripMacBinWindow() {
	delete myMacBinHandler;
}

bool StripMacBinWindow::QuitRequested()
{
        be_app->PostMessage(B_QUIT_REQUESTED);
        return(TRUE);
}

//-----------------
// Set status message to print in main window
//-----------------
void StripMacBinWindow::SetStatus(char *msg)
{
	myView->msg = msg;
	myView->Invalidate();
}

//-----------------
// Start converting MacBinary
//-----------------
void StripMacBinWindow::ConvertFile(record_ref ref)
{
	BFile oldFile;
	BDirectory parentDir;
	oldRef = ref;	// Save for comparison purposes
	char oldFileName[B_FILE_NAME_LENGTH];
	
	switch(myMacBinHandler->open_ref(ref)) {
	case M_NO_ERROR:
		char *newname = myMacBinHandler->get_filename();
		SetStatus("Enter destination filename.");
		// I want the save panel to use the source file's directory
		// as the default for saving the stripped file.
		oldFile.SetRef(ref);
		oldFile.GetName(oldFileName);
		if(!strcmp(oldFileName, newname)) // Old name = MacBinary
			strcat(newname, ".new");		// so add ".new" to end
		oldFile.GetParent(&parentDir);
		BMessage *model = new BMessage(B_SAVE_REQUESTED);
		model->AddRef("directory", parentDir.Record()->Ref());
		RunSavePanel(newname, NULL, NULL, NULL, model);
		break;
	case M_CANT_OPEN:
		SetStatus("Error: Can't open file");
		be_app->PostMessage(READY_FOR_NEXT);
		break;
	case M_READ_ERROR:
		SetStatus("Error: Can't read file");
		be_app->PostMessage(READY_FOR_NEXT);
		break;
	case M_CRC_ERROR:
		SetStatus("Error: Not a MacBinary file!");
		be_app->PostMessage(READY_FOR_NEXT);
		break;
	}
}

//-----------------
// Print cancel message
//-----------------
void StripMacBinWindow::SavePanelClosed(BMessage *message) {
	if (message->FindBool("canceled")) {
		SetStatus("Save canceled.");
		be_app->PostMessage(READY_FOR_NEXT);
	}
}

//-----------------
// Finish converting MacBinary
//-----------------
void StripMacBinWindow::SaveRequested(record_ref dir, const char *filename)
{
	BDirectory saveDir(dir);
	BFile *saveFile = new BFile();
	long errorStatus;
	
	if (saveDir.GetFile(filename, saveFile) < B_NO_ERROR) {
		// File doesn't exist; create it
		if (saveDir.Create(filename, saveFile) < B_NO_ERROR) {
			SetStatus("Error: Can't create destination file");
			delete saveFile;
			be_app->PostMessage(READY_FOR_NEXT);
			return;
		}
	} else {
		// The file panel should handle this, so I don't expect
		// to see this error message.
		SetStatus("Error: File exists!");
	}
	if ((errorStatus=myMacBinHandler->convert(saveFile)) == B_NO_ERROR) {
		SetStatus("Successfully converted!");
	} else if (errorStatus == M_CANT_OPEN) {
		SetStatus("Error: Couldn't open files!");
	} else {
		SetStatus("Error: Source file too short!");
	}
	delete saveFile;
	be_app->PostMessage(READY_FOR_NEXT);
}