/*
	StripMacBinary.cpp
	
	Copyright 1996, Jake Hamby
*/

#include "StripMacBinary.h"
#include "StripMacBinWindow.h"
#include "StripMacBinView.h"

void main() {
	StripMacBinApp *app = new StripMacBinApp();
	app->Run();
	delete app;
}

BRect area(100, 100, 419, 179);  // Where our window goes.

StripMacBinApp::StripMacBinApp() : BApplication('JEH0') {
	reflist = new BList(4);		// List of files to convert
	
	myWindow = new StripMacBinWindow(area);
    myWindow->Show();

    BPopUpMenu *menu = new BPopUpMenu("Main", FALSE, FALSE);
    BMenuItem *item = new BMenuItem("About StripMacBinary...", new BMessage(B_ABOUT_REQUESTED));
    menu->AddItem(item);
    menu->AddSeparatorItem();
	item = new BMenuItem("Open MacBinary...", new BMessage(SHOW_FILE_PANEL), 'O');
	menu->AddItem(item);
	menu->AddSeparatorItem();
    item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
    menu->AddItem(item);

    SetMainMenu(menu);
}

//-----------------------
// Cheezy About box
//-----------------------

void StripMacBinApp::AboutRequested() {
	BAlert *alert;
	
	alert = new BAlert("", "StripMacBinary v0.9\nCopyright 1996, Jake Hamby\n \
							Please send all comments to:\n    jehamby@lightside.com",
				"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);
	alert->Go();
}

//------------------------
// Get command-line args
//------------------------

void StripMacBinApp::ArgvReceived(int argc, char **argv) {
	record_ref *ref;

	for(int i=1; i<argc; i++) {
		// Try to open argv[i]
		ref = new record_ref;
		long err = get_ref_for_path(argv[i], ref);
		if (err) {
			fprintf(stderr, "Error: Can't open file %s\n", argv[i]);
			delete ref;
		} else {
			// Successful, so add to list
			reflist->AddItem(ref);
		}
	}
	StartHandler();
}

//------------------------
// Print message if user canceled file open
//------------------------

void StripMacBinApp::FilePanelClosed(BMessage *message) {
	if (message->FindBool("canceled")) {
		myWindow->Lock();
		myWindow->SetStatus("Open canceled.");
		myWindow->Unlock();
	}
}

//------------------------
// Handle menu message to show file panel
//------------------------
void StripMacBinApp::MessageReceived(BMessage *message) {
	switch (message->what) {
	case SHOW_FILE_PANEL:
		myWindow->Lock();
		myWindow->SetStatus("Choose files to convert.");
		myWindow->Unlock();
		RunFilePanel();		// Use default values
		break;
	case READY_FOR_NEXT:
		StartHandler();		// Send window the next file to convert
		break;
	default:
		BApplication::MessageReceived(message);
		break;
	}
}

//------------------------
// Add refs through drag-n-drop
//------------------------

void StripMacBinApp::RefsReceived(BMessage *message) {
	ulong type;
	long count;
	
	message->GetInfo("refs", &type, &count);
	for(long i=--count; i>=0; i--) {
		record_ref *item = new record_ref;
		*item = message->FindRef("refs", i);
		if (does_ref_conform(*item, "File")) {
			reflist->AddItem(item);
		} else {
			BAlert *alert = new BAlert("", "Please only drag files to this icon!", "OK", NULL, NULL,
					B_WIDTH_AS_USUAL, B_WARNING_ALERT);
			alert->Go();
		}
	}
	StartHandler();
}

//--------------------------
// Give first file to BWindow to strip
// The BWindow can open a SavePanel for the user to choose
// where to save the stripped file.
//--------------------------

void StripMacBinApp::StartHandler() {
	if(reflist->CountItems()) {
		myWindow->Lock();		// Lock window for exclusive use
		record_ref *ref = (record_ref *)reflist->RemoveItem(long(0));
		myWindow->ConvertFile(*ref);
		delete ref;
		myWindow->Unlock();		// Unlock window
	}
}